#include <stdlib.h>
#include <stdio.h>
#include <dmedia/vl.h>

#include "Video.h"

void Video::init()
  {
    /* Connect to the daemon */
    if (!(svr = vlOpenVideo(""))) 
	error_exit();

    /* Set up a drain node in memory */
    drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
    
    /* Set up a source node on any video source  */
    src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);

    /* Create a path using the first device that will support it */
    path = vlCreatePath(svr, VL_ANY, src, drn); 

    /* Set up the hardware for and define the usage of the path */
    if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
	error_exit();

    val.fractVal.numerator = 1;
    val.fractVal.denominator = 4;
    if(vlSetControl(svr, path, drn, VL_ZOOM, &val) < 0)
       vlPerror("Zoom");

    val.xyVal.x = 100;
    val.xyVal.y = 100;
    if(vlSetControl(svr, path, drn, VL_SIZE, &val) < 0)
       vlPerror("Size");

    val.intVal = 5;
    if(vlSetControl(svr, path, drn, VL_RATE, &val) < 0)
       vlPerror("Rate");


    /* Set the packing to RGB */
    val.intVal = VL_PACKING_RGB_8;
    vlSetControl(svr, path, drn, VL_PACKING, &val);
    
    /* Get the video size */
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    xsize = val.xyVal.x;
    ysize = val.xyVal.y;

   /* Create and register a buffer for 1 frame */
   buffer = vlCreateBuffer(svr, path, drn, 1);
   if (buffer == NULL)
      error_exit();	
   vlRegisterBuffer(svr, path, drn, buffer);
   
   /* Begin the data transfer */
   if (vlBeginTransfer(svr, path, 0, NULL))
      error_exit();
  }


char *Video::capture_frame()
  {
     char *frame;

     do {
	sginap(1);		/* wait a tick */
	info = vlGetNextValid(svr, buffer);
     } while (!info);
     
     /* Get a pointer to the frame */
     frame = vlGetActiveRegion(svr, buffer, info);

     return frame;
  }

void Video::frame_release()
  {
     /* Finished with frame, unlock the buffer */
     vlPutFree(svr, buffer);
  }

void Video::close()
  {
   /* End the data transfer */
   vlEndTransfer(svr, path);
   
   /* Cleanup before exiting */
   vlDeregisterBuffer(svr, path, drn, buffer);
   vlDestroyBuffer(svr, buffer);
   vlDestroyPath(svr, path);
   vlCloseVideo(svr);
  }
